## Code written by Frederic Ouimet (February 2022)

require("latex2exp") # to write LaTeX symbols

##########################
# Set the path for plots #
##########################

path <- file.path("C:\\Users\\fred1\\Desktop\\Dossier\\Post-Doctorat\\Latex\\In_preparation\\Ouimet_2022_LLT_matrix_T\\_simulations", fsep = .Platform$file.sep)
setwd(path)

##############
# Parameters #
##############

d <- 2
m <- 2

low_nu <- 5
up_nu <- 1005
inc_nu <- 200
ratio_plot_points <- 1

nus <- seq(low_nu, up_nu, inc_nu)
len_nu <- length(nus)

#############
# Functions #
#############

Delta <- function (X, nu, Sigma, Omega) { # nu > 2, X is a 2x2 matrix, Sigma, Omega > 0 are 2x2 SPD matrices
  inv_Sigma_half <- solve(chol(Sigma))
  inv_Omega_half <- solve(chol(Omega))
  return (inv_Sigma_half %*% X %*% inv_Omega_half)
}

log_f_g <- function (X, nu, Sigma, Omega) { # nu > 2, X is a 2x2 matrix, Sigma, Omega > 0 are 2x2 SPD matrices
  eig <- eigen(Delta(X, nu, Sigma, Omega) %*% t(Delta(X, nu, Sigma, Omega)))$values
  delta <- sqrt((nu - 2) / nu * eig)
  return (- 2 * log((nu - 2) / 2) + (lgamma((nu + 3) / 2) - lgamma((nu + 1) / 2)) +
            (lgamma((nu + 2) / 2) - lgamma(nu / 2)) + 1 / 2 * sum(delta ^ 2) -
            (nu + 3) / 2 * sum(log(1 + (delta / sqrt(nu - 2)) ^ 2)))
}

log_ratio_error <- function (X, nu, Sigma, Omega, ord) { # nu > 2, X is a 2x2 matrix, Sigma, Omega > 0 are 2x2 SPD matrices, c = 0,1,2 is the order of correction
  eig <- eigen(Delta(X, nu, Sigma, Omega) %*% t(Delta(X, nu, Sigma, Omega)))$values
  if (ord == 0) { # Order 0 correction
    return (log_f_g(X, nu, Sigma, Omega))
  } else if (ord == 1) { # Order 1 correction
    return (log_f_g(X, nu, Sigma, Omega) - nu ^ (-1) * (1 / 4 * sum(eig ^ 2) - 5 / 2 * sum(eig) + 5))
  } else { # ord = 2 : Order 2 correction
    res <- return (log_f_g(X, nu, Sigma, Omega) - nu ^ (-1) * (1 / 4 * sum(eig ^ 2) - 5 / 2 * sum(eig) + 5) -
                     nu ^ (-2) * (-1 / 6 * sum(eig ^ 3) + 3 / 4 * sum(eig ^ 2) + 7 / 2))
  }
}

#############
# Main code #
#############

start_time <- Sys.time()

s12 <- 1 # non-diagonal element of S

for (s11 in 2:4) {
  for (s22 in 2:5) {
    Sigma <- matrix(c(s11, s12, s12, s22), nrow = 2)
    Omega <- diag(c(1, 1))
    max_error_vec <- array(0, dim = c(len_nu, 3))
    
    for (i in 1:len_nu) {
      nusi <- nus[i]
      lst <- vector(mode = "list", length = 3)
      lst[[1]] <- seq(-2, 2, nusi ^ (-1 / 2))
      lst[[2]] <- seq(-2, 2, nusi ^ (-1 / 2))
      lst[[3]] <- seq(-2, 2, nusi ^ (-1 / 2))
      hyperrec <- expand.grid(lst) # three dimensional hypercube around 0
      len_hyper <- dim(hyperrec)[1]
      
      for (j in 1:len_hyper) {
        x_hyp_j <- as.numeric(hyperrec[j, ])
        X <- matrix(c(x_hyp_j[1], x_hyp_j[2], x_hyp_j[2], x_hyp_j[3]), nrow = 2)
        
        # we look at the error only if the bulk condition is satisfied
        eig <- eigen(Delta(X, nusi, Sigma, Omega) %*% t(Delta(X, nusi, Sigma, Omega)))$values
        #print(eig)
        #print(Delta(X, nusi, Sigma, Omega) %*% t(Delta(X, nusi, Sigma, Omega)))
        #if((eig[1] < 0) || (eig[2] < 0)) {
        #  stop()
        #}
        if (min(eig) > 0 && max(eig) <= 1) {
          max_error_vec[i, 1] <- max(max_error_vec[i, 1], abs(log_ratio_error(X, nusi, Sigma, Omega, 0)))
          max_error_vec[i, 2] <- max(max_error_vec[i, 2], abs(log_ratio_error(X, nusi, Sigma, Omega, 1)))
          max_error_vec[i, 3] <- max(max_error_vec[i, 3], abs(log_ratio_error(X, nusi, Sigma, Omega, 2)))
        }
      }
    }
    
    mypath <- file.path(path, paste("loglog_errors_plot_", s11, "_", s22, ".pdf", sep = ""))
    pdf(file = mypath)
    par(bg = "white", pty = "s", mgp = c(2, 0.8, 0), mar = c(3.1, 0.8, 1.2, 0.1), cex = 1.8)
    nus_plot_points <- seq(low_nu, up_nu, inc_nu * ratio_plot_points)
    nus_index_plot_points <- seq(1, len_nu, ratio_plot_points)
    plot(nus_plot_points, 1 / max_error_vec[nus_index_plot_points, 1], log = "xy", ylim = c(1, 10 ^ 10), # *** fix this
         col = "blue", pch = 15, xlab = "", ylab = "", cex.lab = 1.3, cex.axis = 1.3)
    points(nus_plot_points, 1 / max_error_vec[nus_index_plot_points, 2], col = "green", pch = 16)
    points(nus_plot_points, 1 / max_error_vec[nus_index_plot_points, 3], col = "red", pch = 17)
    lines(nus, 1 / max_error_vec[, 1], col = "blue", lwd = 2)
    lines(nus, 1 / max_error_vec[, 2], col = "green", lwd = 2)
    lines(nus, 1 / max_error_vec[, 3], col = "red", lwd = 2)
    legend("topleft", legend = c(TeX("$1 / E_0$"), TeX("$1 / E_1$"), TeX("$1 / E_2$")), 
           pch = c(15, 16, 17), lty = c(1, 1, 1), col = c("blue", "green", "red"), pt.cex = 1, cex = 1)
    dev.off()
    
    mypath <- file.path(path, paste("error_exponents_plot_", s11, "_", s22, ".pdf", sep = ""))
    pdf(file = mypath)
    par(bg = "white", pty = "s", mgp = c(2, 0.8, 0), mar = c(3.1, 0.8, 1.2, 0.1), cex = 1.8)
    nus_plot_points <- seq(low_nu, up_nu, inc_nu * ratio_plot_points)
    nus_index_plot_points <- seq(1, len_nu, ratio_plot_points)
    log_nu <- log(nus)
    log_nu_plot_points <- log(nus_plot_points)
    plot(nus_plot_points, -log(max_error_vec[nus_index_plot_points, 1]) / log_nu_plot_points, ylim = c(-0.6, 3.0),
         col = "blue", pch = 15, xlab = "", ylab = "", cex.lab = 1.3, cex.axis = 1.3)
    points(nus_plot_points, -log(max_error_vec[nus_index_plot_points, 2]) / log_nu_plot_points, col = "green", pch = 16)
    points(nus_plot_points, -log(max_error_vec[nus_index_plot_points, 3]) / log_nu_plot_points, col = "red", pch = 17)
    lines(nus, -log(max_error_vec[, 1]) / log_nu, col = "blue", lwd = 2)
    lines(nus, -log(max_error_vec[, 2]) / log_nu, col = "green", lwd = 2)
    lines(nus, -log(max_error_vec[, 3]) / log_nu, col = "red", lwd = 2)
    legend("bottomright", legend = c(TeX("$\\log(E_0) / \\log(\\nu^{-1})$"), TeX("$\\log(E_1) / \\log(\\nu^{-1})$"), TeX("$\\log(E_2) / \\log(\\nu^{-1})$")), 
           pch = c(15, 16, 17), lty = c(1, 1, 1), col = c("blue", "green", "red"), pt.cex = 1, cex = 1)
    dev.off()
    end_time <- Sys.time()
    print(end_time - start_time)
  }
}